// Copyright (C) 2016 EDF
// All Rights Reserved
// This code is published under the GNU Lesser General Public License (GNU LGPL)
#include <boost/version.hpp>
#include "StOpt/core/grids/OneDimRegularSpaceGrid.h"
#include "StOpt/core/grids/OneDimData.h"
#include "StOpt/sddp/OptimizerSDDPBase.h"
#include "test/c++/tools/sddp/OptimizeDemandSDDP.h"
#include "test/c++/tools/simulators/SimulatorGaussianSDDP.h"


#ifdef __linux__
#ifdef __clang__
#if BOOST_VERSION <  105600
// map std::shared ptr to boost python
namespace boost
{
template<class T> T *get_pointer(std::shared_ptr<T> p)
{
    return p.get();
}
}
#endif
#endif
#endif

#include <boost/python.hpp>
#include "StOpt/python/BoostToStdSharedPtr.h"
#include "test/c++/python/FutureCurveWrap.h"

/** \file BoostPythonSDDPOptimizers.cpp
*  \brief permits to map Optimizers for SDDP
* \author Xavier Warin
*/

#ifdef _DEBUG
#undef _DEBUG
#include <Python.h>
#define _DEBUG
#else
#include <Python.h>
#endif
#include <numpy/arrayobject.h>
#include "StOpt/python/NumpyConverter.hpp"




using namespace boost::python;

/// \wrapper for Optimizer for demand test case in SDDP
class OptimizeDemandSDDPWrap : public OptimizeDemandSDDP<SimulatorGaussianSDDP>
{
public :

    /// \brief Constructor
    /// \param   p_sigD                volatility for demand
    /// \param   p_kappaD              AR coefficient for demand
    /// \param   p_timeDAverage        average demand
    /// \param   p_spot                Spot price
    /// \param   p_simulatorBackward   backward  simulator
    /// \param   p_simulatorForward    Forward simulator
    OptimizeDemandSDDPWrap(const  double   &p_sigD, const double &p_kappaD,
                           const  FutureCurve &p_timeDAverage,
                           const double &p_spot,
                           const boost::shared_ptr<SimulatorGaussianSDDP> &p_simulatorBackward,
                           const boost::shared_ptr<SimulatorGaussianSDDP> &p_simulatorForward):
        OptimizeDemandSDDP(p_sigD, p_kappaD,
                           std::make_shared< StOpt::OneDimData< StOpt::OneDimRegularSpaceGrid, double> >(static_cast<StOpt::OneDimData< StOpt::OneDimRegularSpaceGrid, double> >(p_timeDAverage)),
                           p_spot, make_shared_ptr<SimulatorGaussianSDDP>(p_simulatorBackward), make_shared_ptr<SimulatorGaussianSDDP>(p_simulatorForward)) { }
};


// MSVC 2015 BUG
#if (_MSC_VER == 1900)
namespace boost
{
template <>
OptimizeDemandSDDPWrap  const volatile *get_pointer< class  OptimizeDemandSDDPWrap const volatile >(
        class	       OptimizeDemandSDDPWrap  const volatile *c)
{
    return c;
}
}
#endif

BOOST_PYTHON_MODULE(SDDPOptimizers)
{


    Register<Eigen::ArrayXd>();
    Register<Eigen::ArrayXXd>();

    // map optimizer for demand test case
    class_<OptimizeDemandSDDPWrap, std::shared_ptr<OptimizeDemandSDDPWrap>, bases<StOpt::OptimizerSDDPBase> >("OptimizeDemandSDDP",
            init< const  double &, const double &,  const  FutureCurve &,
            const double &,
            const boost::shared_ptr<SimulatorGaussianSDDP> &,
            const boost::shared_ptr<SimulatorGaussianSDDP> &>())
    .def("getSimulatorBackward", &OptimizeDemandSDDP<SimulatorGaussianSDDP>::getSimulatorBackward)
    .def("getSimulatorForward", &OptimizeDemandSDDP<SimulatorGaussianSDDP>::getSimulatorForward)
    .def("oneAdmissibleState", &OptimizeDemandSDDP<SimulatorGaussianSDDP>::oneAdmissibleState)
    ;
}
