// Copyright (C) 2016 EDF
// All Rights Reserved
// This code is published under the GNU Lesser General Public License (GNU LGPL)
#include <boost/version.hpp>
#include <boost/bind.hpp>
#include "StOpt/sddp/SDDPFinalCut.h"
#include "StOpt/sddp/SimulatorSDDPBase.h"
#include "StOpt/sddp/OptimizerSDDPBase.h"
#ifdef __linux__
#ifdef __clang__
#if BOOST_VERSION <  105600
// map std::shared ptr to boost python
namespace boost
{
template<class T> T *get_pointer(std::shared_ptr<T> p)
{
    return p.get();
}
}
#endif
#endif
#endif
#include "boost/python/dict.hpp"
#include "StOpt/python/BoostToStdSharedPtr.h"
#include "StOpt/python/BinaryFileArchiveStOpt.h"

/** \file BoostPythonSDDPUnitTest.cpp
 * \brief test for SDDP mapping
 * \author Xavier Warin
 */

#ifdef _DEBUG
#undef _DEBUG
#include <Python.h>
#define _DEBUG
#else
#include <Python.h>
#endif
#include <numpy/arrayobject.h>
#include "StOpt/python/NumpyConverter.hpp"

using namespace boost::python;

std::pair<double, double> backwardForwardSDDPTestMapping(std::shared_ptr<StOpt::OptimizerSDDPBase> &,
        const int &,
        const Eigen::ArrayXd &,
        const StOpt::SDDPFinalCut &,
        const Eigen::ArrayXd &,
        const Eigen::ArrayXi &,
        const std::string &,
        const std::string &,
        const std::string &,
        int &,
        double &,
        const int &)
{
    return std::make_pair(0., 0.);
}

boost::python::dict backwardForwardSDDPTestMappingWrap(boost::shared_ptr<StOpt::OptimizerSDDPBase>    a,
        const int    &b,
        const Eigen::ArrayXd &c,
        const StOpt::SDDPFinalCut &d,
        const Eigen::ArrayXd &e,
        const Eigen::ArrayXi &f,
        const std::string &g,
        const std::string &h,
        const std::string &i,
        int  j ,
        double  k,
        const int &l)
{
    std::shared_ptr<StOpt::OptimizerSDDPBase>  refLoc = make_shared_ptr<StOpt::OptimizerSDDPBase>(a);
    std::pair<double, double> retLoc = backwardForwardSDDPTestMapping(refLoc, b, c, d, e, f, g, h, i, j, k, l);
    boost::python::dict dicReturn;
    dicReturn["BackWardEstimation"] = retLoc.first;
    dicReturn["ForwardEStimation"] = retLoc.second;
    dicReturn["NumberOfSDDPIterations"] = j;
    dicReturn["AccuracyReached"] = k;
    return dicReturn;

}

/// \brief backwardSDDP wrapper
/// \return value obtained in optimization
double backwardSDDPTestMappingWrap(boost::shared_ptr<StOpt::OptimizerSDDPBase> ,
                                   std::shared_ptr<StOpt::SimulatorSDDPBase> ,
                                   const Eigen::ArrayXd &,
                                   const Eigen::ArrayXd &,
                                   const StOpt::SDDPFinalCut &,
                                   const boost::shared_ptr<BinaryFileArchiveStOpt> &,
                                   const std::string &,
                                   const boost::shared_ptr<BinaryFileArchiveStOpt> &)
{
    return 0. ;
}

double	forwardSDDPTestMappingWrap(boost::shared_ptr<StOpt::OptimizerSDDPBase>   ,
                                   std::shared_ptr<StOpt::SimulatorSDDPBase>  ,
                                   const Eigen::ArrayXd &,
                                   const Eigen::ArrayXd &,
                                   const StOpt::SDDPFinalCut &,
                                   const bool &,
                                   const boost::shared_ptr<BinaryFileArchiveStOpt > &,
                                   const boost::shared_ptr<BinaryFileArchiveStOpt> &,
                                   const std::string &)
{
    return 0. ;
}

class SimulTest : public StOpt::SimulatorSDDPBase
{
public:
    SimulTest() {}
    int getNbSimul() const
    {
        return 0;
    }
    int getNbSample() const
    {
        return 0;
    }
    void updateDates(const double &) {}
    Eigen::VectorXd getOneParticle(const int &) const
    {
        return Eigen::VectorXd::Zero(1);
    }
    Eigen::MatrixXd getParticles() const
    {
        return Eigen::MatrixXd::Zero(1, 1);
    }
    void resetTime() {}
    void updateSimulationNumberAndResetTime(const int &) {}
};


class OptimizeTest : public StOpt::OptimizerSDDPBase
{
    std::shared_ptr< SimulTest> m_simul;

public :
    OptimizeTest() {}
    OptimizeTest(const std::shared_ptr< SimulTest > &p_simul): m_simul(p_simul) {}

    Eigen::ArrayXd oneStepBackward(const std::unique_ptr< StOpt::SDDPCutBase > &, const std::tuple< std::shared_ptr<Eigen::ArrayXd>, int  , int > &, const Eigen::ArrayXd &, const int &) const
    {
        return Eigen::ArrayXd::Zero(1) ;
    }

    double oneStepForward(const Eigen::ArrayXd &, Eigen::ArrayXd &,  Eigen::ArrayXd &, const std::unique_ptr< StOpt::SDDPCutBase > &, const int &) const
    {
        return 0;
    }
    void updateDates(const double &, const double &) {}
    std::shared_ptr<Eigen::ArrayXd> oneAdmissibleState(const double &)
    {
        return std::make_shared<Eigen::ArrayXd>(1);
    }
    int getStateSize() const
    {
        return 0;
    }
    std::shared_ptr< StOpt::SimulatorSDDPBase > getSimulatorBackward() const
    {
        return m_simul;
    }
    std::shared_ptr< StOpt::SimulatorSDDPBase > getSimulatorForward() const
    {
        return m_simul;
    }

};

class OptimizeTestWrap : public OptimizeTest
{
public :
    OptimizeTestWrap() {}
    OptimizeTestWrap(const boost::shared_ptr<  SimulTest > &p_simul): OptimizeTest(make_shared_ptr<SimulTest>(p_simul)) {}
};

// %!@@@ VS2015 update 3 BUG
#if (_MSC_VER == 1900)

namespace boost
{
template <>
OptimizeTestWrap const volatile *get_pointer< class OptimizeTestWrap  const volatile >(
        class OptimizeTestWrap  const volatile *c)
{
    return c;
}
}
#endif

BOOST_PYTHON_MODULE(StOptSDDPUnitTest)
{
    using namespace boost::python;

    Register<Eigen::ArrayXd>();
    Register<Eigen::ArrayXi>();

    /// map test
    class_<SimulTest>("SimulTest")
    .def(init<>())
    ;

    class_<OptimizeTestWrap, std::shared_ptr<OptimizeTestWrap>, bases<StOpt::OptimizerSDDPBase> >("OptimizeTest")
    .def(init< const boost::shared_ptr< SimulTest >& >())
    .def("getSimulatorBackward", &OptimizeTestWrap::getSimulatorBackward)
    ;

    def("backwardForwardSDDPTestMapping", backwardForwardSDDPTestMappingWrap);
    def("backwardSDDPTestMapping", backwardSDDPTestMappingWrap);
    def("forwardSDDPTestMapping", forwardSDDPTestMappingWrap);
}
