// Copyright (C) 2016 EDF
// All Rights Reserved
// This code is published under the GNU Lesser General Public License (GNU LGPL)
#ifndef _BINARYFILEARCHIVESTOPT_H
#define _BINARYFILEARCHIVESTOPT_H
#include <string>
#include "geners/BinaryFileArchive.hh"
#include "geners/Record.hh"
#include "geners/Reference.hh"
#include <boost/shared_ptr.hpp>
#include <boost/python/list.hpp>
#include "StOpt/regression/BaseRegression.h"
#include "StOpt/regression/GridAndRegressedValueGeners.h"
#include "StOpt/core/grids/SpaceGrid.h"
#include "StOpt/core/grids/FullGrid.h"
#include "StOpt/python/BoostToStdSharedPtr.h"
#include "StOpt/python/VectorAndList.h"
#include "StOpt/python/GridAndRegressedValueWrap.h"
#ifdef SDDPPYTHON
#include "StOpt/sddp/LocalLinearRegressionForSDDP.h"
#include "StOpt/sddp/LocalLinearRegressionForSDDPGeners.h"
#include "StOpt/sddp/SDDPVisitedStatesGeners.h"
#include "StOpt/python/SDDPVisitedStatesWrap.h"
#endif

/// \file BinaryFileArchiveStOpt.h
/// \brief defines a wrapping for geners binary file
///        permits to extend the binary archive for StOpt

// create a class to encapsulate a Binary Archive
class BinaryFileArchiveStOpt: public gs::BinaryFileArchive
{


public:

    BinaryFileArchiveStOpt(const char *p_fileName, const char *p_type): gs::BinaryFileArchive(p_fileName, p_type)
    {
    }


    // to use it in StOpt for dump Continuation values
    void dumpGridAndRegressedValue(const std::string &p_name, const int &p_iStep, const  boost::python::list  &p_phiIn,
                                   const   boost::shared_ptr<StOpt::BaseRegression>    &p_condExp,
                                   const boost::shared_ptr< StOpt::SpaceGrid >   &p_grid)
    {
        // convert python to c++ object
        std::vector< std::shared_ptr< Eigen::ArrayXXd > >  phiIn(convertFromListShPtr<Eigen::ArrayXXd>(p_phiIn));
        std::shared_ptr<StOpt::BaseRegression>  condExp =  make_shared_ptr(p_condExp);
        std::shared_ptr<StOpt::FullGrid>  grid = make_shared_ptr<StOpt::FullGrid>(boost::dynamic_pointer_cast<StOpt::FullGrid>(p_grid));

        // values to regress
        std::vector< StOpt::GridAndRegressedValue  > contVal(phiIn.size());
        for (size_t iReg = 0; iReg < phiIn.size(); ++iReg)
            contVal[iReg] =  StOpt::GridAndRegressedValue(grid, condExp, *phiIn[iReg]);
        std::string stepString = boost::lexical_cast<std::string>(p_iStep) ;
        *this << gs::Record(contVal, p_name.c_str(), stepString.c_str()) ;
        this->flush() ; // necessary for python mapping
    }


    // read continuation values
    std::vector< std::shared_ptr<GridAndRegressedValueWrap > >  readGridAndRegressedValue(const int &p_iStep,  const std::string &p_nameCont)
    {
        std::vector< StOpt::GridAndRegressedValue >  continuationObj;
        gs::Reference< std::vector<StOpt::GridAndRegressedValue  > >(*this, p_nameCont.c_str(), boost::lexical_cast<std::string>(p_iStep).c_str()).restore(0, &continuationObj);
        std::vector< std::shared_ptr<GridAndRegressedValueWrap >>  continuationObjWrap(continuationObj.size());
        for (size_t i = 0; i < continuationObjWrap.size(); ++i)
            continuationObjWrap[i] = std::make_shared<GridAndRegressedValueWrap>(continuationObj[i]);
        return continuationObjWrap;
    }

    // dump

#ifdef SDDPPYTHON
    void dumpLocalLinearRegressionForSDDP(const StOpt::LocalLinearRegressionForSDDP &p_sddpReg)
    {
        *this << gs::Record(p_sddpReg, "Regressor", "Top");
        this->flush(); // necessary in python
    }

    void dumpLocalVisitedStateForSDDP(const SDDPVisitedStatesWrap &p_state)
    {
        *this << gs::Record(static_cast<StOpt::SDDPVisitedStates>(p_state), "States", "Top");
        this->flush(); // necessary in python
    }
#endif

};

#endif
