// Copyright (C) 2016 EDF
// All Rights Reserved
// This code is published under the GNU Lesser General Public License (GNU LGPL)
#ifndef GRIDANDREGRESSEDVALUEWRAP_H
#define GRIDANDREGRESSEDVALUEWRAP_H
#include <boost/shared_ptr.hpp>
#include "StOpt/regression/BaseRegression.h"
#include "StOpt/regression/GridAndRegressedValue.h"
#include "StOpt/core/grids/SpaceGrid.h"
#include "StOpt/python/BoostToStdSharedPtr.h"

/** \file GridAndRegressedValueWrap.h
 *  \brief GridAndRegressedValue wrapper used in python mapping
 *  \author Xavier Warin
 */

// wrapper for continuation value : trick use boost::shared_ptr and convert to std::shared_ptr
// unable to wrap correctly so copy
class GridAndRegressedValueWrap: public StOpt::GridAndRegressedValue
{
public :
    GridAndRegressedValueWrap(const boost::shared_ptr< StOpt::SpaceGrid >   &p_grid ,  const boost::shared_ptr<StOpt::BaseRegression>   &p_regressor  ,  const Eigen::ArrayXXd &p_values):
        StOpt::GridAndRegressedValue(make_shared_ptr<StOpt::SpaceGrid>(p_grid), make_shared_ptr<StOpt::BaseRegression>(p_regressor), p_values) {  }

    GridAndRegressedValueWrap(const boost::shared_ptr< StOpt::SpaceGrid >   &p_grid ,  const boost::shared_ptr<StOpt::BaseRegression>   &p_regressor):
        StOpt::GridAndRegressedValue(make_shared_ptr<StOpt::SpaceGrid>(p_grid), make_shared_ptr<StOpt::BaseRegression>(p_regressor)) {  }

    GridAndRegressedValueWrap(const StOpt::GridAndRegressedValue &p_gridAndRegressed):
        StOpt::GridAndRegressedValue(p_gridAndRegressed) {  }

    explicit GridAndRegressedValueWrap(const boost::shared_ptr<StOpt::BaseRegression>   &p_regressor): StOpt::GridAndRegressedValue(make_shared_ptr<StOpt::BaseRegression>(p_regressor)) {}


    double getValue(const Eigen::ArrayXd &p_ptOfStock, const Eigen::ArrayXd &p_coordinates) const
    {
        return StOpt::GridAndRegressedValue::getValue(p_ptOfStock, p_coordinates);
    }

    Eigen::ArrayXXd  getRegressedValues() const
    {
        return StOpt::GridAndRegressedValue::getRegressedValues();
    }

    void setRegressedValues(const Eigen::ArrayXXd &p_regValues)
    {
        StOpt::GridAndRegressedValue::setRegressedValues(p_regValues);
    }

    std::shared_ptr< StOpt::SpaceGrid > getGrid() const
    {
        return  StOpt::GridAndRegressedValue::getGrid();
    }
    std::shared_ptr<StOpt::BaseRegression > getRegressor()const
    {
        return StOpt::GridAndRegressedValue::getRegressor();
    }
};

#endif

